@@ -30,68 +30,57 @@
 #include "dh-settings.h"
 
 typedef struct {
-        DhApp         *application;
-        GtkWidget     *main_box;
         GtkWidget     *view;
         DhSettings    *settings;
-} DhAssistantPriv;
+} DhAssistantPrivate;
 
-static void dh_assistant_class_init (DhAssistantClass *klass);
-static void dh_assistant_init       (DhAssistant      *assistant);
-
-G_DEFINE_TYPE (DhAssistant, dh_assistant, GTK_TYPE_APPLICATION_WINDOW);
-
-#define GET_PRIVATE(instance) G_TYPE_INSTANCE_GET_PRIVATE \
-  (instance, DH_TYPE_ASSISTANT, DhAssistantPriv)
-
-static gboolean
-assistant_key_press_event_cb (GtkWidget   *widget,
-                              GdkEventKey *event,
-                              DhAssistant *assistant)
-{
-        if (event->keyval == GDK_KEY_Escape) {
-                gtk_widget_destroy (GTK_WIDGET (assistant));
-                return TRUE;
-        }
-
-        return FALSE;
-}
+G_DEFINE_TYPE_WITH_PRIVATE (DhAssistant, dh_assistant, GTK_TYPE_APPLICATION_WINDOW);
 
 static void
 assistant_view_open_uri_cb (DhAssistantView *view,
                             const char      *uri,
                             DhAssistant     *assistant)
 {
-        DhAssistantPriv  *priv;
-        GtkWindow* window;
+        DhApp *app;
+        GtkWindow *window;
 
-        priv = GET_PRIVATE (assistant);
-        window = dh_app_peek_first_window (priv->application);
+        app = DH_APP (gtk_window_get_application (GTK_WINDOW (assistant)));
+
+        window = dh_app_peek_first_window (app);
         _dh_window_display_uri (DH_WINDOW (window), uri);
 }
 
 static gboolean
-window_configure_event_cb (GtkWidget *window,
-                           GdkEventConfigure *event,
-                           gpointer user_data)
+dh_assistant_key_press_event (GtkWidget   *widget,
+                              GdkEventKey *event)
 {
-        DhAssistant *assistant;
-        DhAssistantPriv  *priv;
+        if (event->keyval == GDK_KEY_Escape) {
+                gtk_widget_destroy (widget);
+                return TRUE;
+        }
 
-        assistant = DH_ASSISTANT (user_data);
-        priv = GET_PRIVATE (assistant);
-        dh_util_window_settings_save (
-                GTK_WINDOW (assistant),
-                dh_settings_peek_assistant_settings (priv->settings), FALSE);
-	return FALSE;
+        return GTK_WIDGET_CLASS (dh_assistant_parent_class)->key_press_event (widget, event);
+}
+
+static gboolean
+dh_assistant_configure_event (GtkWidget         *widget,
+                              GdkEventConfigure *event)
+{
+        DhAssistant *assistant = DH_ASSISTANT (widget);
+        DhAssistantPrivate *priv = dh_assistant_get_instance_private (assistant);
+
+        dh_util_window_settings_save (GTK_WINDOW (widget),
+                                      dh_settings_peek_assistant_settings (priv->settings),
+                                      FALSE);
+        return GTK_WIDGET_CLASS (dh_assistant_parent_class)->configure_event (widget, event);
 }
 
 static void
-dispose (GObject *object)
+dh_assistant_dispose (GObject *object)
 {
         DhAssistant *assistant = DH_ASSISTANT (object);
-        DhAssistantPriv *priv = GET_PRIVATE (assistant);
-        g_clear_object (&priv->application);
+        DhAssistantPrivate *priv = dh_assistant_get_instance_private (assistant);
+
         g_clear_object (&priv->settings);
 
         G_OBJECT_CLASS (dh_assistant_parent_class)->dispose (object);
@@ -101,58 +90,46 @@ static void
 dh_assistant_class_init (DhAssistantClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
-        g_type_class_add_private (klass, sizeof (DhAssistantPriv));
-        object_class->dispose = dispose;
+        GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+
+        object_class->dispose = dh_assistant_dispose;
+
+        widget_class->key_press_event = dh_assistant_key_press_event;
+        widget_class->configure_event = dh_assistant_configure_event;
+
+        /* Bind class to template */
+        gtk_widget_class_set_template_from_resource (widget_class,
+                                                     "/org/gnome/devhelp/dh-assistant.ui");
+        gtk_widget_class_bind_template_child_private (widget_class, DhAssistant, view);
 }
 
 static void
 dh_assistant_init (DhAssistant *assistant)
 {
-        DhAssistantPriv *priv = GET_PRIVATE (assistant);
+        DhAssistantPrivate *priv = dh_assistant_get_instance_private (assistant);
+
+        gtk_widget_init_template (GTK_WIDGET (assistant));
 
         priv->settings = dh_settings_get ();
-        priv->main_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-        gtk_widget_show (priv->main_box);
-        gtk_container_add (GTK_CONTAINER (assistant), priv->main_box);
-
-        /* i18n: Please don't translate "Devhelp". */
-        gtk_window_set_title (GTK_WINDOW (assistant), _("Devhelp — Assistant"));
-        gtk_window_set_icon_name (GTK_WINDOW (assistant), "devhelp");
-
-        priv->view = dh_assistant_view_new ();
 
         g_signal_connect (priv->view, "open-uri",
                           G_CALLBACK (assistant_view_open_uri_cb),
                           assistant);
 
-        g_signal_connect (assistant, "key-press-event",
-                          G_CALLBACK (assistant_key_press_event_cb),
-                          assistant);
-
-        gtk_box_pack_start (GTK_BOX (priv->main_box),
-                            priv->view, TRUE, TRUE, 0);
-        gtk_widget_show (priv->view);
-
-        dh_util_window_settings_restore (
-                GTK_WINDOW (assistant),
-                dh_settings_peek_assistant_settings (priv->settings), FALSE);
-
-        g_signal_connect (GTK_WINDOW (assistant), "configure-event",
-                          G_CALLBACK (window_configure_event_cb),
-                          assistant);
+        dh_util_window_settings_restore (GTK_WINDOW (assistant),
+                                         dh_settings_peek_assistant_settings (priv->settings),
+                                         FALSE);
 }
 
 GtkWidget *
 dh_assistant_new (DhApp *application)
 {
-        GtkWidget       *assistant;
-        DhAssistantPriv *priv;
+        GtkWidget          *assistant;
+        DhAssistantPrivate *priv;
 
-        assistant = g_object_new (DH_TYPE_ASSISTANT, NULL);
-
-        priv = GET_PRIVATE (assistant);
-        priv->application = g_object_ref (application);
+        assistant = g_object_new (DH_TYPE_ASSISTANT, "application", application, NULL);
 
+        priv = dh_assistant_get_instance_private (DH_ASSISTANT (assistant));
         dh_assistant_view_set_book_manager (DH_ASSISTANT_VIEW (priv->view),
                                             dh_app_peek_book_manager (application));
 
@@ -163,12 +140,12 @@ gboolean
 dh_assistant_search (DhAssistant *assistant,
                      const gchar *str)
 {
-        DhAssistantPriv *priv;
+        DhAssistantPrivate *priv;
 
         g_return_val_if_fail (DH_IS_ASSISTANT (assistant), FALSE);
         g_return_val_if_fail (str != NULL, FALSE);
 
-        priv = GET_PRIVATE (assistant);
+        priv = dh_assistant_get_instance_private (assistant);
 
         if (dh_assistant_view_search (DH_ASSISTANT_VIEW (priv->view), str)) {
                 gtk_widget_show (GTK_WIDGET (assistant));
